# Popular JWT cloud security library patches “remote” code execution hole
## It’s remotely triggerable, but attackers would already have pretty deep network access if they could “prime” your server for compromise.
## Information:

+ Link: https://nakedsecurity.sophos.com/2023/01/10/popular-jwt-cloud-security-library-patches-remote-code-execution-hole/
+ Source: Naked Security
+ Author: Paul Ducklin
+ Date published: 10 January 2023, 19:59
+ Date scraped: 11 January 2023, 01:00

## Article:
![Main article image](https://nakedsecurity.sophos.com/wp-content/uploads/sites/2/2023/01/jwt-1200.png?w=775)


JWT is short for *JSON Web Token*, where JSON itself is short for *JavaScript Object Notation*.


JSON is a modernish way of representing structured data; its format is a bit like XML, and can often be used instead, but without all the opening-and-closing angle brackets to get in the way of legibility.


For example, data that might be recorded like this in XML…



```
<?xml version="1.0" encoding="UTF-8"?>
<data>
   <name>Duck</name>
   <job>
      <employer>Sophos</employer>
      <role>NakSec</role>
   </job>
</data>

```

…might come out like this in JSON:



```
{"name":"Duck","job":{"employer":"Sophos","role":"NakSec"}}

```

Whether the JSON really is easier to read than the XML is an open question, but the big idea of JSON is that because the data is encoded as legal JavaScript source, albeit without any directly or indirectly executable code in it, you can parse and process it using your existing JavaScript engine, like this:


![](https://nakedsecurity.sophos.com/wp-content/uploads/sites/2/2023/01/node-680.png)


The output string `undefined` above merely reflects the fact that `console.log()` is a *procedure* – a function that does some work but doesn’t return a value. The word `Sophos` is printed out as a side-effect of calling the function, while `undefined` denotes what the function calculated and sent back: nothing.


The popularity of JavaScript for both in-browser and server-side programming, plus the visual familiarity of JSON to JavaScript coders, means that JSON is widely used these days, especially when exchanging structured data between web clients and servers.


And one popular use of JSON is the JWT system, which isn’t (officially, at any rate) read aloud as *juh-witt*, as it is written, but peculiarly pronounced *jot*, an English word that is sometimes used to refer the little dot we write above above an `i` or `j`, and that refers to a tiny but potentially important detail.


## Authenticate strongly, then get a temporary token


Loosely speaking, a JWT is a blob of encoded data that is used by many cloud servers as a service access token.


The idea is that you start by proving your identity to the service, for example by providing a username, password and 2FA code, and you get back a JWT.


The JWT sent back to you is a blob of base64-encoded (actually, URL64-encoded) data that includes three fields:


* **Which crytographic algorithm was used** in constructing the JWT.
* **What sort of access the JWT grants**, and for how long.
* **A keyed cryptographic hash of the first two fields**, using a secret key known only to your service provider.


Once you’ve authenticated up front, you can make subsequent requests to the online service, for example to check a product price or to look up an email address in a database, simply by including the JWT in each request, using it as a sort-of temporary access card.


Clearly, if someone steals your JWT after it’s been issued, they can play it back to the relevant server, which will typically give them access instead of you…


…but JWTs don’t need to be saved to disk, usually have a limited lifetime, and are sent and received over HTTPS connections, so that they can’t (in theory at least) easily be sniffed out or stolen.


When JWTs expire, or if they are cancelled for security reasons by the server, you need to go through the full-blown authentication process again in order to re-establish your right to access the service.


But for as long they’re valid, JWTs improve performance because they avoid the need to reauthenticate fully for every online request you want to make – rather like session cookies that are set in your browser while you’re logged into a social network or a news site.


## Security validation as infiltration


Well, cybersecurity news today is full of a revelation by [researchers](https://unit42.paloaltonetworks.com/jsonwebtoken-vulnerability-cve-2022-23529/) at Palo Alto that we’ve variously seen described as a “high-severity flaw” or a “critical security flaw” in a popular JWT implementation.


In theory, at least, this bug could be exploited by cybercriminals for attacks ranging from implanting unauthorised files onto a JWT server, thus maliciously modifying its configuration or modifying the code it might later use, to direct and immediate code execution inside a victim’s network.


Simply put, the act of presenting a JWT to a back-end server for validation – something that typically happens at every API call (jargon for making a service request) – could lead malware being implanted.


But here’s the good news:


* **The flaw isn’t intrinsic to the JWT protocol.** It applies to a specific [implementation](https://www.npmjs.com/package/jsonwebtoken) of JWT called `jsonwebtoken` from a group called Auth0.
* **The bug was patched three weeks ago.** If you’ve [updated your version](https://github.com/auth0/node-jsonwebtoken/releases/tag/v9.0.0) of `jsonwebtoken` from 8.5.1 or earlier to version 9.0.0, which came out on 2022-12-21, you’re now protected from this particular vulnerability.
* **Cybercriminals can’t directly exploit the bug simply by logging in and making API calls.** As far as we can see, although an attacker could subsequently trigger the vulnerability by making remote API requests, the bug needs to be “primed” first by deliberately writing a booby-trapped secret key into your authentication server’s key-store.


According to the researchers, the bug existed in the part of Auth0’s code that validated incoming JWTs against the secret key stored centrally for that user.


As mentioned above, the JWT itself consists of two fields of data denoting your access privileges, and a third field consisting of the first two fields hashed using a secret key known only to the service you’re calling.


To validate the token, the server needs to recalculate the keyed hash of those first two JWT fields, and to confirm the hash that you presented matches the hash it just calculated.


Given that you don’t know the secret key, but you can present a hash that was computed recently using that key…


…the server can infer that you must have acquired the hash from the authentication server in the first place, by proving your identity up front in some suitable way.


## Data type confusion


It turns out that the hash validation code in `jsonwebtoken` assumes (or, until recently, assumed) that the secret key for your account in the server’s own authentication key-store really was a cryptographic secret key, encoded in a standard text-based format such as PEM (short for *privacy enhanced mail*, but mainly used for non-email purposes these days).


If you could somehow corrupt a user’s secret key by replacing it with data that wasn’t in PEM format, but that was, in fact, some other more complex sort of JavaScript data object…


…then you could booby-trap the secret-key-based hash validation calculation by tricking the authentication server into running some JavaScript code of your choice from that infiltrated “fake key”.


Simply put, the server would try to decode a secret key that it assumed was in a format it could handle safely, even if the key wasn’t in a safe format and the server couldn’t deal with it securely.


Note, however, that you’d pretty much need to hack into the secret key-store database first, before any sort of truly remote code execution trigger would be possible.


And if attackers are already able to wander around your network to the point that they can not only poke their noses into but also modify your JWT secret-key database, you’ve probably got bigger problems than **CVE-2022-23539**, as this bug has been designated.


## What to do?


If you’re using an affected version of `jsonwebtoken`, [update to version 9.0.0](https://github.com/auth0/node-jsonwebtoken/releases/tag/v9.0.0) to leave this bug behind.


However, if you’ve now patched but you think crooks might realistically have been able to pull off this sort of JWT attack on your network, patching alone isn’t enough.


In other words, if you think you might have been at risk here, don’t just patch and move on.


Use [threat detection](https://www.sophos.com/en-us/products/endpoint-antivirus/xdr) and [response techniques](https://www.sophos.com/en-us/products/managed-detection-and-response) to look for holes by which cybercriminals could get far enough to attack your network more generally…


…and **make sure you don’t have crooks in your network anyway**, even after applying the patch.




---


**LEARN THE TRICKS THAT CROOKS USE WHEN THEY’RE ALREADY IN**




> [The Active Adversary Playbook 2022](https://news.sophos.com/en-us/2022/06/07/active-adversary-playbook-2022/)
> 
> 








---





## Tags:
***CVE's***: CVE-2022-23539

### Autogenerated tags:
jwt json javascript jwts jsonwebtoken xml cybercriminals api key-store
